#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

import gimp, gimpplugin, math
from gimpenums import *
pdb = gimp.pdb
import gtk, gimpui, gimpcolor
from gimpshelf import shelf

PARNAME = "Selection-Glow-Values"

def debugMessage(Message):
    dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, Message)
    dialog.run()
    dialog.hide()


class reuse_init(object):
        previewLayer = None

        def save_values(self, *values):
                data = []
                for v in values:
                     data.append(str(v))   
                ndata = "|".join(data)
                if not gimp.parasite_find(PARNAME) == None:
                        gimp.parasite_detach(PARNAME)
                gimp.attach_new_parasite(PARNAME, 0, ndata)
                

        def restore_values(self):
                p = gimp.parasite_find(PARNAME)
                if p == None:
                        return None
                return p.data.split("|")

                                
                

        def get_layer_pos(self, layer):
                i = 0
                while i < len(self.img.layers):
                        if layer == self.img.layers[i]:
                                return i
                else:
                        i += 1
                return -1


        def layer_exists(self, layer):
                return layer != None and layer in self.img.layers

        def removePreviews(self):
                if self.layer_exists(self.previewLayer):
                        self.img.remove_layer(self.previewLayer)
                        self.previewLayer = None
                gimp.displays_flush()

        def make_label(self, text):
                label = gtk.Label(text)
                label.set_use_underline(True)
                label.set_alignment(1.0, 0.5)
                label.show()
                return label

        def make_slider_and_spinner(self, init, min, max, step, page, digits):
                controls = {'adj':gtk.Adjustment(init, min, max, step, page), 'slider':gtk.HScale(), 'spinner':gtk.SpinButton()}
                controls['slider'].set_adjustment(controls['adj'])
                controls['slider'].set_draw_value(False)
                controls['spinner'].set_adjustment(controls['adj'])
                controls['spinner'].set_digits(digits)
                controls['slider'].show()
                controls['spinner'].show()
                return controls

        def make_blend_mode_box(self):
            return gimpui.IntComboBox((
              "Normal",        NORMAL_MODE,
              "Dissolve",      DISSOLVE_MODE,
              "Multiply",      MULTIPLY_MODE,
              "Divide",        DIVIDE_MODE,
              "Screen",        SCREEN_MODE,
              "Overlay",       OVERLAY_MODE,
              "Dodge",         DODGE_MODE,
              "Burn",          BURN_MODE,
              "Hard Light",    HARDLIGHT_MODE,
              "Soft Light",    SOFTLIGHT_MODE,
              "Grain Extract", GRAIN_EXTRACT_MODE,
              "Grain Merge",   GRAIN_MERGE_MODE,
              "Difference",    DIFFERENCE_MODE,
              "Addition",      ADDITION_MODE,
              "Subtract",      SUBTRACT_MODE,
              "Darken Only",   DARKEN_ONLY_MODE,
              "Lighten Only",  LIGHTEN_ONLY_MODE,
              "Hue",           HUE_MODE,
              "Saturation",    SATURATION_MODE,
              "Color",         COLOR_MODE,
              "Value",         VALUE_MODE
            ))

        def show_error_msg(self, msg):
                origMsgHandler = pdb.gimp_message_get_handler()
                pdb.gimp_message_set_handler(ERROR_CONSOLE)
                pdb.gimp_message(msg)
                pdb.gimp_message_set_handler(origMsgHandler)

        def stringToColor(self, string):
                colorlist = string[5:-1].split(", ")
                return gimpcolor.RGB(float(colorlist[0]), float(colorlist[1]), float(colorlist[2]), float(colorlist[3]))


class glow(reuse_init):
        def __init__(self, runmode, img, drawable, lmode, inner, color, width, feather, noisef, noises, opacity ):
                self.img = img
                self.drawable = drawable
                self.shelfkey = 'bevel-plugin'
                if runmode == RUN_INTERACTIVE:
                      self.showDialog()
                elif runmode == RUN_NONINTERACTIVE:
                      if inner == 0:
                          innerb = False
                      elif inner == 1:
                          innerb = True

                      fxlayer = self.makeLayer(img, drawable, lmode, innerb, color, width, feather, noisef, noises, opacity )
                elif runmode == RUN_WITH_LAST_VALS:
                    d = self.restore_values()
                    if d == None:                        
                        self.showDialog()
                    else:
                        lmode = int(d[0])
                        if int(d[1]) == 0:
                            innerb = False
                        else:
                            innerb = True
                        red = int(d[2])
                        green = int(d[3])
                        blue = int(d[4])
                        color = gimpcolor.RGB(red,green,blue,255)
                        width = int(d[5])
                        feather = int(d[6])
                        noisef = int(d[7])
                        noises = int(d[8])
                        opacity = float(d[9])

                        fxlayer = self.makeLayer(img, drawable, lmode, innerb, color, width, feather, noisef, noises, opacity )


                        

        def showDialog(self):
                self.dialog = gimpui.Dialog("Selection Glow", "glowdialog")

                self.table = gtk.Table(9, 4, True)
                self.table.set_homogeneous(False)
                self.table.set_row_spacings(8)
                self.table.set_col_spacings(8)
                self.table.show()

#layer mode parameter
                self.lmode_label = self.make_label("_Layer mode:")
                self.table.attach(self.lmode_label, 0, 1, 1, 2)

                self.lmode_box = self.make_blend_mode_box()
                self.lmode_box.set_active(HARDLIGHT_MODE)
                self.lmode_label.set_mnemonic_widget(self.lmode_box)
                self.table.attach(self.lmode_box, 1, 4, 1, 2)
                self.lmode_box.show()

#inner/outer parameter
                self.inner_label = self.make_label("Glow _type:")
                self.table.attach(self.inner_label, 0, 1, 2, 3)

                self.inner_radio = gtk.RadioButton(None, "_Inner glow", True)
                self.outer_radio = gtk.RadioButton(self.inner_radio, "_Outer glow", True)
                self.inner_radio.set_active(True)
                self.outer_radio.set_active(False)

                self.inner_radio.show()
                self.outer_radio.show()
                self.table.attach(self.inner_radio, 1, 2, 2, 3)
                self.table.attach(self.outer_radio, 2, 3, 2, 3)                                

#color parameter
                self.color_label = self.make_label("Glow _colour:")
                self.table.attach(self.color_label, 0, 1, 3, 5)

                self.color_button = gimpui.ColorButton("Colour", 100, 16, gimpcolor.RGB(255,255,255,255))
                self.color_button.set_color(gimpcolor.RGB(255,255,255,255))        
                self.table.attach(self.color_button, 1,2,3,5)
                self.color_label.set_mnemonic_widget(self.color_button)
                self.color_button.show()
                

#width
                self.width_label = self.make_label("Glow _width:")
                self.table.attach(self.width_label, 0, 1, 5, 6)

                self.width_spinner = self.make_slider_and_spinner(10, 1, 300, 1, 10, 0)
                self.width_spinner['adj'].set_value(10)

                self.width_label.set_mnemonic_widget(self.width_spinner['spinner'])
                self.table.attach(self.width_spinner['slider'], 1, 2, 5, 6)
                self.table.attach(self.width_spinner['spinner'], 2, 3, 5, 6)

#feather
                self.feather_label = self.make_label("_Feather:")
                self.table.attach(self.feather_label, 0, 1, 6, 7)

                self.feather_spinner = self.make_slider_and_spinner(10, 0, 600, 1, 10, 0)
                self.feather_spinner['adj'].set_value(10)

                self.feather_label.set_mnemonic_widget(self.feather_spinner['spinner'])
                self.table.attach(self.feather_spinner['slider'], 1, 2, 6, 7)
                self.table.attach(self.feather_spinner['spinner'], 2, 3, 6, 7)

#noisef
                self.noisef_label = self.make_label("_Noise frequency:")
                self.table.attach(self.noisef_label, 0, 1, 7, 8)

                self.noisef_spinner = self.make_slider_and_spinner(1, 1, 8, 1, 5, 0)
                self.noisef_spinner['adj'].set_value(1)

                self.noisef_label.set_mnemonic_widget(self.noisef_spinner['spinner'])
                self.table.attach(self.noisef_spinner['slider'], 1, 2, 7, 8)
                self.table.attach(self.noisef_spinner['spinner'], 2, 3, 7, 8)

#noises
                self.noises_label = self.make_label("Noise _strength:")
                self.table.attach(self.noises_label, 0, 1, 8, 9)

                self.noises_spinner = self.make_slider_and_spinner(0, 0, 255, 1, 10, 0)
                self.noises_spinner['adj'].set_value(0)

                self.noises_label.set_mnemonic_widget(self.noises_spinner['spinner'])
                self.table.attach(self.noises_spinner['slider'], 1, 2, 8, 9)
                self.table.attach(self.noises_spinner['spinner'], 2, 3, 8, 9)


#opacity
                self.opacity_label = self.make_label("O_pacity:")
                self.table.attach(self.opacity_label, 0, 1, 9, 10)

                self.opacity_spinner = self.make_slider_and_spinner(100.0, 0.0, 100.0, 1.0, 10.0, 1)
                self.opacity_spinner['adj'].set_value(100.0)

                self.opacity_label.set_mnemonic_widget(self.opacity_spinner['spinner'])
                self.table.attach(self.opacity_spinner['slider'], 1, 2, 9, 10)
                self.table.attach(self.opacity_spinner['spinner'], 2, 3, 9, 10)                     


                

#end parameters

#see if saved parameters are found, if so restore them
                
                d = self.restore_values()
                #debugMessage(str(d))
                if not d == None:
                        values =[
                                int(d[0]),
                                int(d[1]),
                                int(d[2]),
                                int(d[3]),
                                int(d[4]),
                                int(d[5]),
                                int(d[6]),
                                int(d[7]),
                                int(d[8]),
                                float(d[9])                                
                                ]
                        
                        self.lmode_box.set_active(values[0])
                        
                        if values[1] == 1:
                                self.inner_radio.set_active(True)
                        elif values[1] == 0:
                                self.outer_radio.set_active(True)

                        self.color_button.set_color(gimpcolor.RGB(values[2],values[3],values[4]))
                                                    
                        self.width_spinner['adj'].set_value(values[5])
                        self.feather_spinner['adj'].set_value(values[6])
                        self.noisef_spinner['adj'].set_value(values[7])
                        self.noises_spinner['adj'].set_value(values[8])
                        self.opacity_spinner['adj'].set_value(values[9])
                        
      
                        
#buttons
                            
                self.dialog.vbox.hbox1 = gtk.HBox(False, 7)
                self.dialog.vbox.hbox1.show()
                self.dialog.vbox.pack_start(self.dialog.vbox.hbox1, True, True, 7)
                self.dialog.vbox.hbox1.pack_start(self.table, True, True, 7)

                reset_button = gtk.Button("Reset")
                reset_button.connect("clicked", self.resetbutton)
                reset_button.show()

                self.preview_button = gtk.Button("Preview")
                self.preview_button.connect("clicked", self.preview)
                self.preview_button.set_size_request(100, -1)
                self.preview_button.show()

 #               if gtk.alternative_dialog_button_order():
 #                       ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
 #                       cancel_button = self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
 #                       self.dialog.action_area.add(reset_button)
 #                       self.dialog.action_area.add(self.preview_button)
 #               else:
                self.dialog.action_area.add(reset_button)
                self.dialog.action_area.add(self.preview_button)
                cancel_button = self.dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
                ok_button = self.dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
                
                ok_button.connect("clicked", self.okbutton)
                cancel_button.connect("clicked", self.cancelbutton)
                self.dialog.show()
                self.dialog.run()
                self.removePreviews()
    
        def cancelbutton(self, widget):
                if self.layer_exists(self.previewLayer):
                        self.img.remove_layer(self.previewLayer)
                        self.previewLayer = None        
        def okbutton(self, widget):
#                debugMessage("I got here")                                
                # remove old preview layer if it exists
                if self.layer_exists(self.previewLayer):
                        self.img.remove_layer(self.previewLayer)
                        self.previewLayer = None
                
                # get values of the settings
                lmode = self.lmode_box.get_active()
                inner = self.inner_radio.get_active()
                color = self.color_button.get_color()
                width = int(self.width_spinner['adj'].get_value())
                feather = int(self.feather_spinner['adj'].get_value())
                noisef = int(self.noisef_spinner['adj'].get_value())
                noises = int(self.noisef_spinner['adj'].get_value())
                opacity = self.opacity_spinner['adj'].get_value()


#save values to parasite

                if inner:
                    innerv = 1
                else:
                    innerv = 0

                
                self.save_values(lmode,innerv,color[0],color[1],color[2],width,feather,noisef,noises,opacity)
                
                # then build new layer with current settings to actually impliment...

                fxlayer = self.makeLayer(self.img, self.drawable, lmode, inner, color, width, feather, noisef, noises, opacity)

                


#resetbutton

                
        def resetbutton(self, widget):
                self.lmode_box.set_active(HARDLIGHT_MODE)

                self.inner_radio.set_active(True)
                self.width_spinner['adj'].set_value(10.0)
                self.feather_spinner['adj'].set_value(10.0)
                self.color_button.set_color(gimpcolor.RGB(255,255,255,255))

                self.noisef_spinner['adj'].set_value(1)
                self.noises_spinner['adj'].set_value(0)
                
                self.opacity_spinner['adj'].set_value(100.0)

##previewbutton             


        def preview(self, widget):
                ptxt = self.preview_button.get_label()

                if self.layer_exists(self.previewLayer):
                        self.img.remove_layer(self.previewLayer)
                        gimp.displays_flush()

                else:
                        lmode = self.lmode_box.get_active()
                        inner = self.inner_radio.get_active()

                        color = self.color_button.get_color()
                        width = int(self.width_spinner['adj'].get_value())
                        feather = int(self.feather_spinner['adj'].get_value())
                        noisef = int(self.noisef_spinner['adj'].get_value())
                        noises = int(self.noises_spinner['adj'].get_value())
                        opacity = self.opacity_spinner['adj'].get_value()
                         
                        self.previewLayer = self.makeLayer(self.img, self.drawable, lmode, inner, color, width, feather, noisef, noises, opacity)
#todo                        

                if ptxt == "Preview":
                        ptxt = "Undo Preview"
                else:
                        ptxt = "Preview"
                self.preview_button.set_label(ptxt)

#start effect

        def makeLayer(self, img, drawable, lmode, inner, color, width, feather, noisef, noises, opacity):                

                
                gimp.context_push()
                img.undo_group_start()

        #create the glow layer
                if inner:
                        lname = "Inner Glow"
                else:
                        lname = "Outer Glow"
                newlayer = gimp.Layer(img, lname, img.width, img.height, RGBA_IMAGE, opacity, lmode)       
                img.add_layer(newlayer, -1)
                  
        #check if selection is empty and invert if so
                if pdb.gimp_selection_is_empty(img):
                    pdb.gimp_selection_invert(img)
                

        #save current selection
                tmpchan = pdb.gimp_selection_save(img)      
                pdb.gimp_context_set_foreground(color)

#+++ INNER GLOW

                
                if inner:
                     pdb.gimp_edit_bucket_fill(newlayer,0,0,100,255,0,0,0)
                     pdb.gimp_selection_feather(img, feather)
                     pdb.gimp_selection_shrink(img, width)
                     if not pdb.gimp_selection_is_empty(img):
                          pdb.gimp_edit_clear(newlayer)
                     pdb.gimp_selection_load(tmpchan)
                     if noises>0:
                         pdb.plug_in_hsv_noise(img, newlayer, (9-noisef), 0, 0, noises)
#+++ OUTER GLOW                    
                else:
                     pdb.gimp_selection_feather(img, feather)
                     pdb.gimp_selection_grow(img, width)
                     pdb.gimp_edit_bucket_fill(newlayer,0,0,100,255,0,0,0)
                     if noises>0:
                         pdb.plug_in_hsv_noise(img, newlayer, (9-noisef), 0, 0, noises)
                     pdb.gimp_selection_load(tmpchan)
                     if not pdb.gimp_selection_is_empty(img):
                          pdb.gimp_edit_clear(newlayer)

        #remove temp channel
                img.remove_channel(tmpchan)


        #end undo group and finish plugin
                gimp.context_pop()
                img.undo_group_end()                        
                pdb.gimp_displays_flush()
                return newlayer


class pyGlow(gimpplugin.plugin):
        def start(self):
                gimp.main(self.init, self.quit, self.query, self._run)

        def init(self):
                pass

        def quit(self):
                pass

        def query(self):
                authorname = "dd"
                copyrightname = "dd"
                menu_location = "<Image>/File/Create/dd/Glow"
                date = "2011"
                plug_description = "Selection glow v1.5"
                plug_help = "Applies a coloured glow in the shape of the selection"
                plug_params = [
                        (PDB_INT32, "run_mode", "Run mode"),
                        (PDB_IMAGE, "image", "Input image"),
                        (PDB_DRAWABLE, "drawable", "Input drawable"),
                        ####### 3 params above needed by all scripts using gimpplugin.plugin ######################

#glow parameters - radio buttons, boxes
                        (PDB_INT32, "lmode", "Layer mode"),                        
                        (PDB_INT32, "inner", "Type"),   # 1=inner 0=outer

#sliders
                        (PDB_COLOR, "color", "Glow colour"),
                        (PDB_FLOAT, "width", "Glow width"),
                        (PDB_FLOAT, "feather", "Feather"),                        
                        (PDB_FLOAT, "noisef", "Noise frequency"),
                        (PDB_FLOAT, "noises", "Noise strength"),
                        (PDB_FLOAT, "opacity", "Layer opacity")
                        ]

                gimp.install_procedure("python_fu_glow",
                        plug_description,
                        plug_help,
                        authorname,
                        copyrightname,
                        date,
                        menu_location,
                        "RGB*",
                        PLUGIN,
                        plug_params,
                        [])

        def python_fu_glow(self, runmode, img, drawable, lmode=HARDLIGHT_MODE, inner=1, color=gimpcolor.RGB(255,255,255,255), width=10, feather=10, noisef=0, noises=0, opacity=100 ):
                glow(runmode, img, drawable, lmode, inner, color, width, feather, noisef, noises, opacity)

if __name__ == '__main__':
        pyGlow().start()
